=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2011-09-06 13:35:07 +0000
+++ bzrlib/builtins.py	2011-09-08 12:29:08 +0000
@@ -5741,18 +5741,30 @@
     takes_args = ['location?']
     takes_options = [
         RegistryOption.from_kwargs(
-            'target_type',
-            title='Target type',
-            help='The type to reconfigure the directory to.',
+            'tree_type',
+            title='Tree type',
+            help='The relation between branch and tree.',
             value_switches=True, enum_switch=False,
             branch='Reconfigure to be an unbound branch with no working tree.',
             tree='Reconfigure to be an unbound branch with a working tree.',
             checkout='Reconfigure to be a bound branch with a working tree.',
             lightweight_checkout='Reconfigure to be a lightweight'
                 ' checkout (with no local history).',
+            ),
+        RegistryOption.from_kwargs(
+            'repository_type',
+            title='Repository type',
+            help='Location fo the repository.',
+            value_switches=True, enum_switch=False,
             standalone='Reconfigure to be a standalone branch '
                 '(i.e. stop using shared repository).',
             use_shared='Reconfigure to use a shared repository.',
+            ),
+        RegistryOption.from_kwargs(
+            'repository_trees',
+            title='Trees in Repository',
+            help='Whether new branches in the repository have trees.',
+            value_switches=True, enum_switch=False,
             with_trees='Reconfigure repository to create '
                 'working trees on branches by default.',
             with_no_trees='Reconfigure repository to not create '
@@ -5772,9 +5784,9 @@
             ),
         ]
 
-    def run(self, location=None, target_type=None, bind_to=None, force=False,
-            stacked_on=None,
-            unstacked=None):
+    def run(self, location=None, bind_to=None, force=False,
+            tree_type=None, repository_type=None, repository_trees=None,
+            stacked_on=None, unstacked=None):
         directory = bzrdir.BzrDir.open(location)
         if stacked_on and unstacked:
             raise errors.BzrCommandError("Can't use both --stacked-on and --unstacked")
@@ -5785,33 +5797,44 @@
         # At the moment you can use --stacked-on and a different
         # reconfiguration shape at the same time; there seems no good reason
         # to ban it.
-        if target_type is None:
+        if (tree_type is None and
+            repository_type is None and
+            repository_trees is None):
             if stacked_on or unstacked:
                 return
             else:
                 raise errors.BzrCommandError('No target configuration '
                     'specified')
-        elif target_type == 'branch':
+        reconfiguration = None
+        if tree_type == 'branch':
             reconfiguration = reconfigure.Reconfigure.to_branch(directory)
-        elif target_type == 'tree':
+        elif tree_type == 'tree':
             reconfiguration = reconfigure.Reconfigure.to_tree(directory)
-        elif target_type == 'checkout':
+        elif tree_type == 'checkout':
             reconfiguration = reconfigure.Reconfigure.to_checkout(
                 directory, bind_to)
-        elif target_type == 'lightweight-checkout':
+        elif tree_type == 'lightweight-checkout':
             reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
                 directory, bind_to)
-        elif target_type == 'use-shared':
+        if reconfiguration:
+            reconfiguration.apply(force)
+            reconfiguration = None
+        if repository_type == 'use-shared':
             reconfiguration = reconfigure.Reconfigure.to_use_shared(directory)
-        elif target_type == 'standalone':
+        elif repository_type == 'standalone':
             reconfiguration = reconfigure.Reconfigure.to_standalone(directory)
-        elif target_type == 'with-trees':
+        if reconfiguration:
+            reconfiguration.apply(force)
+            reconfiguration = None
+        if repository_trees == 'with-trees':
             reconfiguration = reconfigure.Reconfigure.set_repository_trees(
                 directory, True)
-        elif target_type == 'with-no-trees':
+        elif repository_trees == 'with-no-trees':
             reconfiguration = reconfigure.Reconfigure.set_repository_trees(
                 directory, False)
-        reconfiguration.apply(force)
+        if reconfiguration:
+            reconfiguration.apply(force)
+            reconfiguration = None
 
 
 class cmd_switch(Command):

=== modified file 'bzrlib/tests/blackbox/test_reconfigure.py'
--- bzrlib/tests/blackbox/test_reconfigure.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/blackbox/test_reconfigure.py	2011-09-06 18:37:06 +0000
@@ -21,9 +21,10 @@
     workingtree,
     )
 from bzrlib.branchbuilder import BranchBuilder
-
-
-class TestReconfigure(tests.TestCaseWithTransport):
+from bzrlib.tests.script import TestCaseWithTransportAndScript
+
+
+class TestReconfigure(TestCaseWithTransportAndScript):
 
     def test_no_type(self):
         branch = self.make_branch('branch')
@@ -175,15 +176,45 @@
         self.run_bzr('revert', working_dir='checkout')
         self.check_file_contents('checkout/file', 'foo\n')
 
-    def test_lightweight_knit_checkout_to_tree(self, format=None):
+    def test_lightweight_knit_checkout_to_tree(self):
         self.test_lightweight_format_checkout_to_tree('knit')
 
-    def test_lightweight_pack092_checkout_to_tree(self, format=None):
+    def test_lightweight_pack092_checkout_to_tree(self):
         self.test_lightweight_format_checkout_to_tree('pack-0.92')
 
-    def test_lightweight_rich_root_pack_checkout_to_tree(self, format=None):
+    def test_lightweight_rich_root_pack_checkout_to_tree(self):
         self.test_lightweight_format_checkout_to_tree('rich-root-pack')
 
+    def test_branch_and_use_shared(self):
+        self.run_script("""\
+$ bzr init -q branch
+$ echo foo > branch/foo
+$ bzr add -q branch/foo
+$ bzr commit -q -m msg branch
+$ bzr init-repo -q .
+$ bzr reconfigure --branch --use-shared branch
+$ bzr info branch
+Repository branch (format: ...)
+Location:
+  shared repository: .
+  repository branch: branch
+""")
+
+    def test_use_shared_and_branch(self):
+        self.run_script("""\
+$ bzr init -q branch
+$ echo foo > branch/foo
+$ bzr add -q branch/foo
+$ bzr commit -q -m msg branch
+$ bzr init-repo -q .
+$ bzr reconfigure --use-shared --branch branch
+$ bzr info branch
+Repository branch (format: ...)
+Location:
+  shared repository: .
+  repository branch: branch
+""")
+
 
 class TestReconfigureStacking(tests.TestCaseWithTransport):
 

